# Table.py -- Pamhyr
# Copyright (C) 2023  INRAE
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.

# -*- coding: utf-8 -*-

import logging
import traceback

from tools import trace, timer

from PyQt5.QtCore import (
    Qt, QVariant, QAbstractTableModel,
    QCoreApplication, QModelIndex, pyqtSlot,
    QRect,
)

from PyQt5.QtWidgets import (
    QDialogButtonBox, QPushButton, QLineEdit,
    QFileDialog, QTableView, QAbstractItemView,
    QUndoStack, QShortcut, QAction, QItemDelegate,
    QComboBox,
)

from View.Tools.PamhyrTable import PamhyrTableModel

from View.SolverParameters.UndoCommand import *

from Solver.Solvers import solver_long_name, solver_type_list

logger = logging.getLogger()

_translate = QCoreApplication.translate


class TableModel(PamhyrTableModel):
    def _setup_lst(self):
        self._tab = self._opt_data
        self._lst = self._data.get_params(self._tab)

    def data(self, index, role):
        if role != Qt.ItemDataRole.DisplayRole:
            return QVariant()

        row = index.row()
        column = index.column()
        cname = self._headers[column]

        if cname == "name":
            value = self._lst.get(row)[cname]
            if value in self._trad.get_dict("names"):
                value = self._trad.get_dict("names")[value]
            return value
        elif cname == "value":
            value = self._lst.get(row)[cname]
            if value in self._trad.get_dict("yes_no"):
                value = self._trad.get_dict("yes_no")[value]
            return value

        return QVariant()

    def setData(self, index, value, role=Qt.EditRole):
        if not index.isValid() or role != Qt.EditRole:
            return False

        row = index.row()
        column = index.column()

        try:
            if self._headers[column] == "value":
                if value in self._trad.get_dict("r_yes_no"):
                    value = self._trad.get_dict("r_yes_no")[value].lower()

                self._undo.push(
                    SetCommand(
                        self._lst, row,
                        "value", value
                    )
                )
        except Exception as e:
            logger.info(e)
            logger.debug(traceback.format_exc())

        self.dataChanged.emit(index, index)
        return True

    # def add(self, row, parent=QModelIndex()):
    #     self.beginInsertRows(parent, row, row - 1)

    #     self._undo.push(
    #         AddCommand(
    #             self._lst, row
    #         )
    #     )

    #     self.endInsertRows()
    #     self.layoutChanged.emit()

    # def delete(self, rows, parent=QModelIndex()):
    #     self.beginRemoveRows(parent, rows[0], rows[-1])

    #     self._undo.push(
    #         DelCommand(
    #             self._lst, rows
    #         )
    #     )

    #     self.endRemoveRows()
    #     self.layoutChanged.emit()

    # def sort(self, _reverse, parent=QModelIndex()):
    #     self.layoutAboutToBeChanged.emit()

    #     self._undo.push(
    #         SortCommand(
    #             self._lst, False
    #         )
    #     )

    #     self.layoutAboutToBeChanged.emit()
    #     self.layoutChanged.emit()

    # def move_up(self, row, parent=QModelIndex()):
    #     if row <= 0:
    #         return

    #     target = row + 2

    #     self.beginMoveRows(parent, row - 1, row - 1, parent, target)

    #     self._undo_stack.push(
    #         MoveCommand(
    #             self._lst, "up", row
    #         )
    #     )

    #     self.endMoveRows()
    #     self.layoutChanged.emit()

    # def move_down(self, index, parent=QModelIndex()):
    #     if row > len(self._lst):
    #         return

    #     target = row

    #     self.beginMoveRows(parent, row + 1, row + 1, parent, target)

    #     self._undo_stack.push(
    #         MoveCommand(
    #             self._lst, "down", row
    #         )
    #     )

    #     self.endMoveRows()
    #     self.layoutChanged.emit()

    def undo(self):
        self._undo.undo()
        self.layoutChanged.emit()

    def redo(self):
        self._undo.redo()
        self.layoutChanged.emit()
